/*
** Module   : WebClientBuilder.java
** Abstract : build and start a platform process for web chui client.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140206 First version, extracted from ClientBuilder.
** 002 MAG 20140711 Enable/Disable clipboard.
*/

package com.goldencode.p2j.main;

import java.util.*;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.net.*;

/**
 * This class is used to create and spawn a web client process using a {@link ProcessBuilder}
 * instance.  A special small tools are designed for Linux / Windows OS in order to spawn the 
 * process on the OS user account.  Both tools are written in C using specific OS API and are 
 * build as executable using ant native target.  The source code and the make file for this tools 
 * are available in the /src/native process directory.  The password is provided vis stdin 
 * redirection on both platforms.  The command line parameters for the spawned process are 
 * constructed from default parameters stored in directory and some specific parameters stored 
 * inside a {@link WebClientBuilderParameters} structure.
 */
public class WebClientBuilder
extends ClientBuilder
{   
   /** Server key store */
   private ServerKeyStore serverKeyStore;
   
   /**
    * Constructor.
    * 
    * @param   config
    *          The specific parameters for this client.
    */
   public WebClientBuilder(ClientBuilderParameters config)
   {
      super(config);

      BootstrapConfig serverConfig = SessionManager.get().config();
      
      // load server key store
      serverKeyStore = new ServerKeyStore(serverConfig);
   }
 
   /**
    * Get server key store
    * 
    * @return  The loaded server key store.
    */
   public ServerKeyStore getServerKeyStore()
   {
      return (serverKeyStore != null && serverKeyStore.isValid()) ? serverKeyStore : null;
   }

   /**
    * Add terminal options.
    */
   protected void addClientOptions()
   {
      WebClientBuilderParameters webConfig = (WebClientBuilderParameters) config;
      Map<String, String> terminalOptions = webConfig.getTerminalOptions();

      // add to command
      command.add("client:driver:type=" + terminalOptions.get("type"));
      command.add("client:chui:rows=" + terminalOptions.get("rows"));
      command.add("client:chui:columns=" + terminalOptions.get("columns"));
      command.add("client:chui:background=" + terminalOptions.get("background"));
      command.add("client:chui:foreground=" + terminalOptions.get("foreground"));
      command.add("client:chui:selection=" + terminalOptions.get("selection"));
      command.add("client:chui:fontname=" + terminalOptions.get("fontname"));
      command.add("client:chui:fontsize=" + terminalOptions.get("fontsize"));
      command.add("client:web:socketTimeout=" + terminalOptions.get("webSocketTimeout"));
      command.add("client:web:watchdogTimeout=" + terminalOptions.get("watchdogTimeout"));
      if (terminalOptions.get("port") != null)
      {
         command.add("client:web:port=" + terminalOptions.get("port"));
      }
      if (terminalOptions.get("host") != null)
      {
         command.add("client:web:host=" + terminalOptions.get("host"));
      }

      // Add referrer address
      command.add("web:referrer:url=" + webConfig.getReferrer());
      // Clipboard enable/disable
      command.add("client:web:clipboard=" + terminalOptions.get("clipboard"));
   }

   /**
    * Get the spawn arguments.  Web chui clients call the {@code spawn} tool using the
    * {@code spawn 0 <user> <workdir> <command> [args]} syntax.
    * <p>
    * As these arguments are already built into the {@link ClientBuilder#command} list, it just
    * returns the already build list with a "1" prefix. 
    * 
    * @return   See above.
    */
   @Override
   protected List<String> getSpawnArguments()
   {
      List<String> cmd = new ArrayList<>();
      
      // add mode
      cmd.add("1");
      // the entire command
      cmd.addAll(command);
      
      return cmd;
   }
 }
