/*
** Module   : WebClientBuilderOptions.java
** Abstract : store chuiWeb parameters read from directory.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140206 First version, extracted from ClientBuilderOptions.
** 002 MAG 20140711 Enable/Disable clipboard.
*/

package com.goldencode.p2j.main;

import java.util.*;

import com.goldencode.p2j.net.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.chui.driver.swing.*;

/**
 * The parameters stored inside this structure are read from directory on server start-up 
 * and are used as default parameters to create and spawn a web client process.
 * <p>
 * This class MUST NOT BE USED from client side, as it uses a direct access to the server.  Also,
 * this class can be instantiated only from threads with a P2J context for the running P2J server.
 */
public class WebClientBuilderOptions
extends ClientBuilderOptions
{   
   /** Directory node name, {@code "chuiWeb"} string. */
   private static final String DIRECTORY_NODE_ID = "chuiWeb";

   /**
    * Singleton instance. Not set as final to allow it to be invalidated (in case these
    * options are changed via admin console).
    */
   private static WebClientBuilderOptions instance = null;

   /** Terminal option **/
   private final Map<String, String> terminalOptions = new HashMap<>();

   /**
    * Initialize. Read all configuration parameters from directory.
    * 
    * @throws   IllegalStateException
    *           If this is called from P2J clients.
    */
   private WebClientBuilderOptions()
   {
      super(null);
      
      if (SessionManager.get() == null || SessionManager.get().isLeaf())
      {
         throw new IllegalStateException("This class can be used only on the server-side!");
      }

      // Read Terminal Options
      readTerminalOptions();
   }
   
   /**
    * Get {@link WebClientBuilderOptions} singleton instance.
    */
   public static WebClientBuilderOptions getInstance()
   {
      // TODO: this will not allow changing the options via the AdminConsole - add a way to 
      // invalidate and recreate this instance

      // this needs to be initialized always in a thread with the server's context

      if (instance == null)
      {
         instance = new WebClientBuilderOptions();
      }
      return instance;
   }

   /**
    * Get Terminal Options.
    * 
    * @return  The terminal options.
    */
   public Map<String, String> getTerminalOptions()
   {
      return terminalOptions;
   }

   /**
    * Get the authentication mode for this spawner.
    * 
    * @return   Always {@code true}.
    */
   protected boolean isPasswordAuthentication()
   {
      return true;
   }

   /**
    * Read terminal options.
    */
   private void readTerminalOptions()
   {
      // default terminal options
      int rows = TerminalOptions.DEF_ROWS;
      int columns = TerminalOptions.DEF_COLUMNS;
      // getRGB places alpha on bits 24-31, which are emitted as FF by default. this are not
      // interpreted properly by the browser.
      String background = "0x" + Integer.toHexString(TerminalOptions.DEF_BACKGROUND.getRGB())
                                        .substring(2);
      String foreground = "0x" + Integer.toHexString(TerminalOptions.DEF_FOREGROUND.getRGB())
                                        .substring(2);
      String selection = "0x" + Integer.toHexString(TerminalOptions.DEF_SELECTION.getRGB())
                                        .substring(2);
      String fontname = TerminalOptions.DEF_FONTNAME;
      int fontsize = TerminalOptions.DEF_FONTSIZE;
      long stimeout = -1;
      long wtimeout = -1;
      int port = 0;
      String host = null;
      boolean clipboard = true; 

      // get registered terminal options
      rows       = getNode(DIRECTORY_NODE_ID, "rows", rows);
      columns    = getNode(DIRECTORY_NODE_ID, "columns", columns);
      background = getNode(DIRECTORY_NODE_ID, "background", background);
      foreground = getNode(DIRECTORY_NODE_ID, "foreground", foreground);
      selection  = getNode(DIRECTORY_NODE_ID, "selection", selection);
      fontname   = quotedString(getNode(DIRECTORY_NODE_ID, "fontname", fontname));
      fontsize   = getNode(DIRECTORY_NODE_ID, "fontsize", fontsize);
   
      stimeout   = getNode(DIRECTORY_NODE_ID, "webSocketTimeout", (int) stimeout);
      wtimeout   = getNode(DIRECTORY_NODE_ID, "watchdogTimeout",  (int) wtimeout);
      port       = getNode(DIRECTORY_NODE_ID, "port", port);
      host       = getNode(DIRECTORY_NODE_ID, "host", host);
      clipboard  = getNode(DIRECTORY_NODE_ID, "clipboard", true);
   
      // build terminal options map
      terminalOptions.put("type", OutputManager.CHUI_DRIVER_WEB);
      terminalOptions.put("rows", String.valueOf(rows));
      terminalOptions.put("columns", String.valueOf(columns));
      terminalOptions.put("background", background);
      terminalOptions.put("foreground", foreground);
      terminalOptions.put("selection", selection);
      terminalOptions.put("fontname", fontname);
      terminalOptions.put("fontsize", String.valueOf(fontsize));
      terminalOptions.put("webSocketTimeout", String.valueOf(stimeout));
      terminalOptions.put("watchdogTimeout", String.valueOf(stimeout));
      terminalOptions.put("port", String.valueOf(port));
      terminalOptions.put("host", host);
      terminalOptions.put("clipboard", clipboard ? "true" : "false");
   }
}
