/*
** Module   : ChuiWebPageHandler.java
** Abstract : delivers main page to tests
**
** Copyright (c) 2013-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131105 Created initial version.
** 002 MAG 20140114 Add new replacements.
** 003 MAG 20140217 Add referrer replacement.
** 004 MAG 20140303 Add authorization token.
** 005 MAG 20140711 Enable/Disable clipboard.
*/

package com.goldencode.p2j.ui.client.chui.driver.web;

import java.io.*;
import java.util.logging.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.eclipse.jetty.server.*;

import com.goldencode.p2j.cfg.*;
import com.goldencode.p2j.main.*;
import com.goldencode.p2j.util.*;
import com.goldencode.p2j.web.*;

/**
 * Jetty Handler which delivers HTML resource files.
 * This class provide a method to parse the HTML text and
 * substitute place holders. 
 */
public class ChuiWebPageHandler
extends HtmlResourceHandler
{
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(ChuiWebPageHandler.class.getName());
      
   /** Configuration. */
   private BootstrapConfig config;
   
   /** Authorization token */
   private String authorizationToken;   
   
   /**
    * Constructor.
    *
    * @param    config
    *           Configuration parameters.
    * @param    targetRoot
    *           Prefix for all URLs relating to this page.
    * @param    skeletonPage
    *           Fully qualified resource name of the HTML 
    *           file which should be used.
    */
   public ChuiWebPageHandler(BootstrapConfig config, String targetRoot, String skeletonPage)
   {
      super(targetRoot, skeletonPage);
      this.config = config;
            
      LOG.logp(Level.INFO,
               "ChuiWebPageHandler",
               "",
               LogHelper.generate("target=%s page=%s", targetRoot, skeletonPage));
   }
   
   /**
    * Return the main page for AJAX clients. If authorization required and not authorized
    * do not load the main page client will get a HTTP 404 error, page not found.
    *    
    * @param   target 
    *          The target of the request - either a URI or a name.
    * @param   base
    *          The base request.
    * @param   request
    *          The object or a wrapper of the request.
    * @param   response 
    *          The object or a wrapper of the response. 
    */
   public void handle(String              target, 
                      Request             base,
                      HttpServletRequest  request, 
                      HttpServletResponse response)
   throws IOException, 
          ServletException
   {
      if (isAuthorized(base))
      {
         super.handle(target, base, request, response);
      }    
   }
   
   /**
    * Set authorization token.
    * 
    * @param   token
    *          Authorization token;
    */
   public void setAuthorizationToken(String token)
   {
      this.authorizationToken = token;
   }
   
   /**
    * Process the given string and make any replacements of parameter
    * values as needed. The replacement placeholder follow the JEE 
    * expression language (EL) format for immediate evaluation. 
    * <p>
    * Example: ${expression} 
    *
    * @param    base
    *           The details and state the request.
    * @param    text
    *           The text line that may need modifications.
    *
    * @return   The new text if a modification occurred or the original text
    *           if no modification was needed.
    */
   protected String handleReplacements(Request base, String text)
   {
      if (text.contains("${context}"))
      {
         text = text.replace("${context}", base.getContextPath());
      }
      
      if (text.contains("${rows}"))
      {
         int rows = config.getInt("client", "chui", "rows", 24);
         text = text.replace("${rows}", String.valueOf(rows));
      }
      
      if (text.contains("${cols}"))
      {
         int cols = config.getInt("client", "chui", "columns", 80);
         text = text.replace("${cols}", String.valueOf(cols));
      }
      
      if (text.contains("${font.name}"))
      {
         String fontName = config.getString("client", "chui", "fontname", "monospace");
         text = text.replace("${font.name}", String.valueOf(fontName));
      }
      
      if (text.contains("${font.size}"))
      {
         int fontSize = config.getInt("client", "chui", "fontsize", 12);
         text = text.replace("${font.size}", String.valueOf(fontSize));
      }

      if (text.contains("${font.color}"))
      {
         String fontColor = config.getString("client", "chui", "foreground", "#FFA500");
         text = text.replace("${font.color}", htmlColor(fontColor));
      }

      if (text.contains("${font.background}"))
      {
         String fontBackground = config.getString("client", "chui", "background", "#000000");
         text = text.replace("${font.background}", htmlColor(fontBackground));
      }

      if (text.contains("${referrer}"))
      {
         String referrer = config.getString("web", "referrer", "url", null);               
         text = text.replace("${referrer}", referrer);
      }
      
      if (text.contains("${clipboard.enabled}"))
      {
         boolean enabled = config.getBoolean("client", "web", "clipboard", true);               
         text = text.replace("${clipboard.enabled}", enabled ? "true" : "false");
      }

      return text;
   }
   
   /**
    * Convert to HTML color format.
    * 
    * @param   color
    *          Color text string.
    * @return  HTML format color
    */
   private String htmlColor(String color)
   {
      return color.startsWith("0x") ? color.replace("0x", "#") : color;
   }

   /**
    * Check authorization token. Check the browser cookies list for a cookie which contains an 
    * authorization token that match the requested token. If found the requester is authorized.
    * 
    * @param   request
    *          HttpServletRequest instance.
    *          
    * @return  <code>true</code> if authorized <code>false</code> otherwise.
    */
   private boolean isAuthorized(HttpServletRequest request)
   {
      if (authorizationToken != null)
      {
         // get browser cookies
         Cookie[] cookies = request.getCookies();
      
         if (cookies != null)
         {
            // search for authorization cookie
            for (Cookie cookie : cookies)
            {
               String name = cookie.getName();
               String value = cookie.getValue();
               if (WebHandler.AUTHORIZATION_TOKEN.equals(name) && authorizationToken.equals(value))
               {
                  return true;
               }
            }
         }
      }
      
      return false;
   }
}
